/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: chatwindow.cpp,v 1.6 2001/01/17 02:01:54 bills Exp $
// $Log: chatwindow.cpp,v $
// Revision 1.6  2001/01/17 02:01:54  bills
// port kicq to new chat/file session interface, add icqlib subdir
//
// Revision 1.5  2000/12/19 06:17:06  bills
// change icqlink member in regwin and mainwindow to an ICQLINK *, use
// icq_ICQLINKNew and icq_ICQLINKDelete instead of icq_Init and icq_Done
//
// Revision 1.4  2000/10/10 14:46:16  davidvh
// Replaced the CTRL-<ENTER> shortcut for 'Send' with ALT-S.
// Made the shortcut for cancel ALT-C.
// Made variable names easier to read and fixed my styling to match Denis'.
//
// Revision 1.3  2000/10/02 07:37:42  denis
// First two patches from David van Hoose <david-vh@home.com> with some
// modifications were applied.
//
// Revision 1.2  2000/09/13 19:11:12  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.1.2.1  2000/07/22 19:44:03  napalmbox
// kicq compiles now with -DQT_NO_ASCII_CAST
//
// Revision 1.1  2000/05/24 16:56:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.12  2000/02/07 03:01:21  bills
// added code to understand icq chat escape sequences
//
// Revision 1.11  2000/01/10 16:08:06  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.10  1999/12/02 13:58:35  denis
// Cleanups.
// IcqWindow random segfaults fixed.
//
// Revision 1.9  1999/10/14 12:46:18  denis
// Cleanups.
//
// Revision 1.8  1999/09/29 17:30:51  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:47:43  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:01:00  denis
// Internalization support added.
//
// Revision 1.5  1999/07/12 15:23:18  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.4  1999/06/30 13:58:52  bills
// cleanups, added resize functionality
//
// Revision 1.3  1999/04/17 20:22:13  bills
// modified for new icqwindow base class
//
// Revision 1.2  1999/04/14 15:36:04  denis
// Cleanups.
//
// Revision 1.1  1999/04/04 21:57:12  bills
// added for chat support
//

#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kiconloader.h>

#include "chatwindow.h"
#include "mainwindow.h"
#include "chatedit.h"
#include "wrapedit.h"
#include "chatedit.h"

#include "chatwindow.moc"

ChatWindow::ChatWindow(IcqChatSession *session) 
  : IcqWindow(), icqChatSession(session)
{
  CrashGuard();
  splitter = new QSplitter(QSplitter::Vertical, this);
  splitter->setOpaqueResize(true);

  localBox = new ChatEdit(splitter);
  localBox->setLimit(0);

  remoteBox = new WrapEdit(splitter);

  localBox->setFocus();

  resize(500, 250);
  setMinimumSize(400, 250);

  splitter->resize(500, 250);

  connect(icqChatSession, SIGNAL(closed()), SLOT(sessionClosed()));
  connect(icqChatSession, SIGNAL(data(int length, char *data)),
    SLOT(textReceived(int length, char *data)));
  connect(localBox, SIGNAL(data(QString data)), 
    SLOT(textEntered(QString data)));
}

ChatWindow::~ChatWindow()
{
  RemovePointer(remoteBox);
  RemovePointer(localBox);
  RemovePointer(splitter);
}

void ChatWindow::CrashGuard(void)
{
  splitter = 0;
  localBox = 0;
  splitter = 0;
  return;
}

void ChatWindow::sessionClosed()
{
  QMessageBox::information(this, i18n("ICQ Chat"), i18n("Last user has left chat"));

  if (!requestComplete())
  {
    m_requestComplete = true;
    close();
  }
}

void ChatWindow::textEntered(QString data)
{
  icqChatSession->sendData(data.data());
}

void ChatWindow::textReceived(int length, char *data)
{
  int row,col,i;

  for(i=0; i < length;)
  {
    QString qs;
    char ch, *p;

    remoteBox->cursorPosition(&row, &col);
    p = ((char *)data) + i++;
    ch = *p++;
    switch(ch)
    {
      case '\x00': /* Set text color - RR GG BB 00 */
        {
          QPalette mypalette = (remoteBox->palette()).copy();
          QColorGroup cgrp = mypalette.normal();
          QColor c (*(unsigned char *)p, *(unsigned char *)(p+1), *(unsigned char *)(p+2));
          QColorGroup ncgrp (c, cgrp.background(), cgrp.light(), cgrp.dark(), cgrp.mid(), c, cgrp.base());
          mypalette.setNormal(ncgrp);
          mypalette.setDisabled(ncgrp);
          mypalette.setActive(ncgrp);
          remoteBox->setPalette(mypalette);
          remoteBox->setBackgroundColor(cgrp.base());
        }
        i += 4;
        break;
      case '\x01': /* Set background color - RR GG BB 00 */
        {
          QPalette mypalette = (remoteBox->palette()).copy();
          QColorGroup cgrp = mypalette.normal();
          QColor c (*(unsigned char *)p, *(unsigned char *)(p+1), *(unsigned char *)(p+2));
          QColorGroup ncgrp (cgrp.foreground(), cgrp.background(), cgrp.light(), cgrp.dark(), cgrp.mid(), cgrp.text(), c);
          mypalette.setNormal(ncgrp);
          mypalette.setDisabled(ncgrp);
          mypalette.setActive(ncgrp);
          remoteBox->setPalette(mypalette);
          remoteBox->setBackgroundColor(c);
        }
        i += 4;
        break;
      case '\x11': /* Font style (Bold=1, Italic=2, Underline=4) */
        {
          QFont f = remoteBox->font();
          int curattr = 0;
          int newattr = *(int *)p;
          int chgattr;
          if(f.bold())
            curattr |= 1;
          if(f.italic())
            curattr |= 2;
          if(f.underline())
            curattr |= 4;
          if((chgattr = curattr ^ newattr) != 0)
          {
            if(chgattr & 1)
              f.setBold(newattr & 1);
            if(chgattr & 2)
              f.setItalic(newattr & 2);
            if(chgattr & 4)
              f.setUnderline(newattr & 4);
            remoteBox->setFont(f);
          }
        }
        i += 4;
        break;
      case '\x12': /* Font point size */
        {
          QFont f = remoteBox->font();
          if(f.pointSize() != *(int *)p)
          {
            f.setPointSize(*(unsigned int *)p);
            remoteBox->setFont(f);
          }
        }
        i += 4;
        break;
      case 16: /* Font family and encoding */
        {
          QFont f = remoteBox->font();

          if(strcasecmp(f.family().latin1(), p + 2) != 0)
          {
            f.setFamily(p + 2);
            remoteBox->setFont(f);
          }
        }
        i += *p + 2 + 2;
        break;
      case 3: /* Chat is active */
      case 4: /* Chat is inactive */
      case '\b':   /* Backspace */
        if(col > 0)
        {
          /* Very ugly but works */
          bool _autoUpdate = remoteBox->autoUpdate();
          char *line = strdup(remoteBox->textLine(row).latin1());
          strcpy(&line[col - 1], &line[col]);
          remoteBox->setAutoUpdate(FALSE);
          remoteBox->removeLine(row);
          remoteBox->insertLine(line, row);
          remoteBox->setCursorPosition(row, col - 1);
          remoteBox->setAutoUpdate(_autoUpdate);
        }
        break;
      case 7: /* Bell */
        kapp->beep();
        break;
      default:
        qs.fill(((ch == 13) ? 10 : ch), 1);
        remoteBox->insertAt(qs, row, col);
    }
  }
  return;
}

void ChatWindow::closeEvent(QCloseEvent *ce)
{
  bool complete = requestComplete();

  IcqWindow::closeEvent(ce);

  if (!complete)
    icqChatSession->close();

  return;
}

void ChatWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  splitter->setGeometry(0, 0, width(), height());
  return;
}
