/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: separatoritem.cpp,v 1.4 2001/01/29 05:20:03 denis Exp $
// $Log: separatoritem.cpp,v $
// Revision 1.4  2001/01/29 05:20:03  denis
// Patch by Ben Reser <ben@reser.org> to correctly redraw Contact List
// separator has been applied.
//
// Revision 1.3  2000/10/14 21:00:50  davidvh
// Minor fixes.
//
// Revision 1.2  2000/09/13 19:11:12  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.1.2.2  2000/07/11 16:36:59  napalmbox
// some add/fixe ...
//
// Revision 1.1.2.1  2000/07/02 05:45:54  napalmbox
// port kicq -> kde2 :))
//
// Revision 1.1  2000/06/17 17:38:03  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//

#include <kapp.h>
#include <kiconloader.h>
#include <qlistbox.h>
#include "separatoritem.h"
#include "mainwindow.h"
#include "userlist.h"

SeparatorItem::SeparatorItem(sepType t) : VirtUserItem()
{
  itype = t;
  switch(t)
  {
    case tpOnSeparator:
      setText(i18n("Online"));
      break;
    case tpOffSeparator:
      setText(i18n("Offline"));
      break;
    case tpNILSeparator:
      setText(i18n("Not In List"));
      break;
  }
  return;
}

SeparatorItem::~SeparatorItem(void)
{
  return;
}

void SeparatorItem::paint(QPainter *p)
{
  QFont f = p->font();
  QFontMetrics fm = p->fontMetrics();

  int w = MW->listWidget()->width()-5;
  p->eraseRect(0, 0, w, fm.lineSpacing());
  p->setPen(QColor(Qt::white));
  p->drawLine(0, fm.lineSpacing()/2, w/2-fm.width(text())/2-6, fm.lineSpacing()/2);
  p->drawLine(w/2+fm.width(text())/2+5, fm.lineSpacing()/2, w, fm.lineSpacing()/2);
  p->setPen(QColor(Qt::black));
  p->drawLine(0, fm.lineSpacing()/2-1, w/2-fm.width(text())/2-6, fm.lineSpacing()/2-1);
  p->drawLine(w/2+fm.width(text())/2+5, fm.lineSpacing()/2-1, w, fm.lineSpacing()/2-1);
  p->drawText(0, 0, w, fm.lineSpacing(), Qt::AlignCenter, text());
  return;
}

int SeparatorItem::height(const QListBox *lb) const
{
  return lb->fontMetrics().lineSpacing();
}

int SeparatorItem::width(const QListBox *lb) const
{
  return lb->fontMetrics().width(text());
}

SeparatorItem::sepType SeparatorItem::type(void)
{
  return itype;
}
