/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _FILEWINDOW_
#define _FILEWINDOW_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qdatetime.h>
#include <time.h>
#include <qsplitter.h>
#include <qlineedit.h>
#include <qprogressbar.h>
#include <qslider.h>
#include <qbutton.h>

#include <icq.h>

#include "icqwindow.h"
#include "icqfilesession.h"

#define PROGRESS_SCALE       1024.0

class FileWindow : public IcqWindow {
  Q_OBJECT
public:
  FileWindow(IcqFileSession *);
  ~FileWindow(void);
  void filePacket(int);
  void setWorkingDirectory(const QString &directory);
  void acceptFileRequest(void);
public slots:
  void statusChanged(int status);
  void progress(unsigned long current, unsigned long total);
  void updateTimes(void);
signals:
protected:
  unsigned long remote_uin;
  unsigned long m_id;
  IcqFileSession *icqFileSession;
  QString working_directory;
  void resizeEvent(QResizeEvent *);
  void closeEvent(QCloseEvent *);
  QString formatTime(long seconds);
protected slots:
  void abortClicked(void);
  void sessionClosed();
private:
  // GUI
  QGridLayout *mainLayout;
  // Header
  QGridLayout *headerLayout;
  QLabel *fileLabel, *fileCountLabel, *operationLabel, *statusLabel;
  QLineEdit *headerFileName, *fileCount, *operationFileName, *statusText;
  // Bar
  QFrame *line1, *line2;
  // Progress section
  QGridLayout *midLayout;
  QLabel *progressLabel, *progressLabel2;
  QProgressBar *progressBar, *batchProgressBar;
  QLabel *sizeLabel1, *sizeLabel2;
  QLineEdit *fileSizeText, *batchSizeText;
  QLabel *elapsedLabel, *remainingLabel, *cpsLabel;
  QLineEdit *elapsedText, *remainingText, *cpsText;
  // Bottom
  QGridLayout *bottomLayout;
  QLabel *speedLabel;
  QSlider *speedSlider;
  QPushButton *skipBtn, *abortBtn;

//  QLineEdit *fileText;
//  QLineEdit *status;
  // Status
  time_t cps_timer, cps_start_time;
  unsigned long start_bytes;
  double total_scale, current_scale, cps;
  signed long elapsed_time;

  void CrashGuard(void);
private slots:
  void sliderChanged(int new_value);
};
#endif /* _FILEWINDOW_ */
