/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * $Id: mainconfig.cpp,v 1.16 2001/08/05 23:42:03 davidvh Exp $
 * $Log: mainconfig.cpp,v $
 * Revision 1.16  2001/08/05 23:42:03  davidvh
 * Modified code so the main window sets its size and position before
 * appearing on the screen.
 *
 * Revision 1.15  2001/05/23 23:57:02  denis
 * Warning cleanup.
 *
 * Revision 1.14  2000/11/29 01:56:18  bills
 * Applied Ben Reser's <breser@users.sourceforge.net> patches:
 *
 * #102246 - remember window position and size
 * #102261 - use KLocale when formatting date and time
 * #102262 - Fixes History Display
 *
 * Thanks Ben!
 *
 * Revision 1.13  2000/10/14 21:00:50  davidvh
 * Minor fixes.
 *
 * Revision 1.12  2000/10/02 22:48:32  napalmbox
 * little fix
 *
 * Revision 1.11  2000/10/02 21:44:06  kir
 * New UConfig code.
 *
 * Revision 1.10  2000/10/02 07:37:47  denis
 * First two patches from David van Hoose <david-vh@home.com> with some
 * modifications were applied.
 *
 * Revision 1.9  2000/10/01 18:42:39  kir
 * Florian, you should have only one UConfig instance pointed to db file. DB could be corrupted during code like:
 *
 * mainWindow::loadConfig(...)
 * {
 * 	config = new UConfig(...);
 * ...
 * }
 *
 * Then you have many functions like:
 *
 * savePasswordconfig(..)
 * {
 * 	config = new UConfig(..);
 * delete config;
 * }
 *
 * Ok. There are a lot of warnings, I just try to show you right way of uconfig usage. Correct code please.
 *
 * Revision 1.8  2000/09/30 19:46:17  napalmbox
 * big update
 *
 * Revision 1.7  2000/09/14 03:05:41  napalmbox
 * sounds fixed
 *
 * Revision 1.5  2000/09/14 01:04:34  napalmbox
 * sound fixed
 *
 * Revision 1.4  2000/09/13 23:28:54  napalmbox
 * *.lo ignore , static libs -> noinst shared libs
 *
 * Revision 1.3  2000/09/13 19:21:42  napalmbox
 * replace
 *
 * Revision 1.1.2.3  2000/09/02 21:41:12  napalmbox
 * fix fix fix ....a big fix
 *
 * Revision 1.1.2.2  2000/07/22 19:40:30  napalmbox
 * kicq now compiles with -DQT_NO_ASCII_CAST ...
 *
 * Revision 1.4  2000/07/11 18:33:29  denis
 * saveConfigSound() now receives one more parameter "clear".
 * Cleanups.
 *
 * Revision 1.3  2000/07/10 09:24:24  pcadach
 * Recording of first-time registration information updated. See ChangeLog.
 *
 * Revision 1.2  2000/07/09 18:59:09  denis
 * Multi-owner feature was added.
 * Note: move your ~/.kde/share/config/kicqrc to
 * ~/.kde/share/apps/kicq/owners/<UIN>.conf and create
 * ~/.kde/share/config/kicqrc with section [Owner] and key UIN=<UIN>
 * where <UIN> is your current UIN (ICQ#) -- kicq-upgrade.sh will changed
 * later.
 *
 * Revision 1.1  2000/06/17 16:49:36  denis
 * Whole mainwindow.cpp was splitted for several smaller files:
 * mainconfig.cpp, specialops.cpp and, of course, mainwindow.cpp
 *
 */

#include <qdir.h>
#include <ksimpleconfig.h>

#include "mainwindow.h"
#include "icqsound.h"
#include "userlist.h"


bool MainWindow::checkConfig(void)
{
  return (bool)getOwnerConfigUIN();
}

QString MainWindow::getOwnerPath(unsigned long u)
{
  return QString("/Owners/") + QString::number(u);
}

QString MainWindow::getOwnerPath(void)
{
  return getOwnerPath(getOwnerConfigUIN());
}

unsigned long MainWindow::getOwnerConfigUIN(void)
{
  db->setPath("/Owners");
  Q_UINT32 u;
  if(!db->readEntry("Owner UIN", u))
    u = 0;
  return u;
}

void MainWindow::setOwnerConfigUIN(unsigned long u)
{
  db->setPath("/Owners");
  db->writeEntry("Owner UIN", (Q_UINT32)u);
  db->flush(true);
  return;
}

icqStatus MainWindow::loadConfig(void)
{
  uin = getOwnerConfigUIN();
  passwd = getPassword();

  QString bp = getOwnerPath();
  
  // Read nick.
  db->setPath(bp + "/Main");
  nick = db->readEntry("Nick").toString();

  // Read server prefferences
  db->setPath("../Preferences/Connections/ICQ Server");
  server = db->readEntry("Host").toString();
  port   = db->readEntry("Port").toInt();

  db->setPath("../Proxy");
  useproxy  = db->readEntry("Use").toBool();
  proxyhost = db->readEntry("Host").toString();
  proxyport = db->readEntry("Port").toInt();
  proxyauth = db->readEntry("Auth").toBool();
  proxyuser = db->readEntry("User").toString();
  proxypass = db->readEntry("Password").toString();

  // Read sound prefferences
  db->setPath(bp + "/Preferences/Events/Sound");
  playWav = db->readEntry("playWav").toBool();

  // Read misc settings
  db->setPath(bp + "/Preferences/Misc");
  icq_Russian  = db->readEntry("Koi2win").toBool();
  closeRecv    = db->readEntry("CloseRecv").toBool();
  convertCRLF  = db->readEntry("ConvertCRLF").toBool();
  quote        = db->readEntry("Quote").toBool();
  reconnect    = db->readEntry("Reconnect").toBool();
  icqStatus st = (icqStatus)db->readEntry("Status").toInt();
  setAllOnline(db->readEntry("AllOnline").toBool());

  // Log settings
  db->setPath(bp + "/Preferences/Events/Log");
  icq_LogLevel = db->readEntry("Level").toInt();
  logfile      = db->readEntry("File").toString();

  // Status settings
  db->setPath(bp + "/Preferences/Auto-Away");
  autoawaytimeout = db->readEntry("Away").toInt();
  autonatimeout   = db->readEntry("NA").toInt();
  autoawayenabled = db->readEntry("Enable").toBool();
  autoawaytona    = db->readEntry("AwayToNA").toBool();

  // Window settings
  db->setPath(bp + "/Geometry/MainWindow");

  int mw = db->readEntry("width").toInt();
  int mh = db->readEntry("height").toInt();
  int mx = db->readEntry("x").toInt();
  int my = db->readEntry("y").toInt();
  setMinimumSize(QSize(140, 300));
  resize(QSize(mw, mh));
  move(QPoint(mx, my));
  if(db->readEntry("Restored").toBool())
    show();

  // Load sounds
  MW->sound->loadsounds();
  return st;
}

void MainWindow::saveConfigOwner(unsigned long u, const QString &nick, const QString &pass)
{
  db->setPath(getOwnerPath(u) + "/Main");
  db->writeEntry("Nick", nick);
  setPassword(pass);
  db->flush(true);
  return;
}

void MainWindow::saveConfigOwner(const QString &nick, const QString &pass)
{
  saveConfigOwner(getOwnerConfigUIN(), nick, pass);
  return;
}

void MainWindow::saveConfigServer(unsigned long u, const QString &host, unsigned short port)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Connections/ICQ Server");
  db->writeEntry("Host", host);
  db->writeEntry("Port", port);
  db->flush(true);
  return;
}

void MainWindow::saveConfigServer(const QString &host, unsigned short port)
{
  saveConfigServer(getOwnerConfigUIN(), host, port);
  return;
}

void MainWindow::saveConfigProxy(unsigned long u, int pruse, const QString &prhost,
                                 unsigned short prport, int prauth, const QString &pruser,
                                 const QString &prpass)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Connections/Proxy");

  db->writeEntry("Use", pruse);
  db->writeEntry("Host", prhost);
  db->writeEntry("Port", (int)prport);
  db->writeEntry("Auth", prauth);
  db->writeEntry("User", pruser);
  db->writeEntry("Password", prpass);
  db->flush();
  return;
}

void MainWindow::saveConfigProxy(int pruse, const QString &prhost, unsigned short prport,
                                 int prauth, const QString &pruser, const QString &prpass)
{
  saveConfigProxy(getOwnerConfigUIN(), pruse, prhost, prport, prauth, pruser, prpass);
  return;
}

void MainWindow::saveConfigLog(unsigned long u, int loglvl, const QString &logfile)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Events/Log");

  icq_LogLevel = loglvl;

  db->writeEntry("Level",loglvl);
  db->writeEntry("File",logfile);
  db->flush(true);
  return;
}

void MainWindow::saveConfigLog(int loglvl, const QString &logfile)
{
  saveConfigLog(getOwnerConfigUIN(), loglvl, logfile);
  return;
}

void MainWindow::saveConfigSound(unsigned long u, int wav, bool clear)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Events/Sound");

  db->writeEntry("playWav", wav);

  if(clear)
    sound->clear();

  MW->sound->savesounds();
  return;
}

void MainWindow::saveConfigSound(int wav, bool clear)
{
  saveConfigSound(getOwnerConfigUIN(), wav, clear);
  return;
}

void MainWindow::saveConfigAway(unsigned long u, int awaytimeout, int natimeout, bool awayenabled,
                                bool awaytona)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Auto-Away");
  db->writeEntry("Away", awaytimeout);
  autoawaytimeout = awaytimeout;
  db->writeEntry("NA", natimeout);
  autonatimeout = natimeout;
  db->writeEntry("Enable", awayenabled);
  autoawayenabled = awayenabled;
  db->writeEntry("AwayToNA", awaytona);
  autoawaytona = awaytona;
  db->flush(true);
  return;
}

void MainWindow::saveConfigAway(int awaytimeout, int natimeout, bool awayenabled, bool awaytona)
{
  saveConfigAway(getOwnerConfigUIN(), awaytimeout, natimeout, awayenabled, awaytona);
  return;
}

void MainWindow::saveConfigOther(unsigned long u, int rus, 
                                 int closereply, int crlf, 
                                 int quoteorig, int recon)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Misc");

  icq_Russian = rus;

  db->writeEntry("Koi2win",icq_Russian);
  db->writeEntry("CloseRecv",closereply);
  db->writeEntry("ConvertCRLF",crlf);
  db->writeEntry("Quote",quoteorig);
  db->writeEntry("Reconnect",recon);
  db->writeEntry("Status",(int)cur_status);
  db->flush(true);
  return;
}

void MainWindow::saveConfigOther(int rus, int closereply, int crlf, int quoteorig, int recon)
{
  saveConfigOther(getOwnerConfigUIN(), rus, closereply, crlf, quoteorig, recon);
  return;
}

void MainWindow::saveProperties(unsigned long u)
{
  db->setPath(getOwnerPath(u) + "/Geometry/MainWindow");
  db->writeEntry("Restored", (int)isVisible());
  db->writeEntry("width", MW->width());
  db->writeEntry("height", MW->height());
  db->writeEntry("x",MW->x());
  db->writeEntry("y",MW->y());
  db->flush(true);
  return;
}

void MainWindow::saveProperties(void)
{
  saveProperties(getOwnerConfigUIN());
  return;
}

void MainWindow::saveStatus(unsigned long u)
{
  db->setPath(getOwnerPath(u) + "/Preferences/Misc");
  db->writeEntry("Status", (unsigned int)cur_status);
  db->writeEntry("AllOnline",listwidget->getAllOnline());
  db->flush(true);
  return;
}

void MainWindow::saveStatus(void)
{
  saveStatus(getOwnerConfigUIN());
  return;
}

QString MainWindow::getPassword(void)
{
  db->setPath(getOwnerPath() + "/Main");

  QString oldpasswd = db->readEntry("Password").toString();

  if(!oldpasswd.isEmpty())
  {
    db->writeEntry("EPassword", encryptStr(oldpasswd));
    db->writeEntry("Password", "");
    db->flush(true);
  }
  return decryptStr(db->readEntry("EPassword").toString());
}

void MainWindow::setPassword(QString password)
{
  db->setPath(getOwnerPath() + "/Main");
  db->writeEntry("EPassword", encryptStr(password));
  db->writeEntry("Password", "");
  db->flush(true);
  return;
}

void MainWindow::saveSessionConfig(void)
{
  saveProperties();
  saveStatus();
  return;
}

QString MainWindow::encryptStr(const QString aStr)
{
  if(aStr.isEmpty())
    return "";

  unsigned int i;
  unsigned int len = aStr.length();
  QCString result;
  result.resize(len+1);

  for (i=0; i<len; i++)
    result[i] = (char)(255 - aStr[i] + ' ');
  result[i] = '\0';
  return result;
}

QString MainWindow::decryptStr(const QString aStr)
{
  return encryptStr(aStr);
}
