/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: findwindow.cpp,v 1.10 2000/12/19 06:17:06 bills Exp $
// $Log: findwindow.cpp,v $
// Revision 1.10  2000/12/19 06:17:06  bills
// change icqlink member in regwin and mainwindow to an ICQLINK *, use
// icq_ICQLINKNew and icq_ICQLINKDelete instead of icq_Init and icq_Done
//
// Revision 1.9  2000/11/29 01:56:18  bills
// Applied Ben Reser's <breser@users.sourceforge.net> patches:
//
// #102246 - remember window position and size
// #102261 - use KLocale when formatting date and time
// #102262 - Fixes History Display
//
// Thanks Ben!
//
// Revision 1.8  2000/10/07 22:53:04  napalmbox
// clean up layout->activate() and "magic" resize(1,1) - not need for QT2.2.X up
//
// Revision 1.7  2000/10/02 21:44:06  kir
// New UConfig code.
//
// Revision 1.6  2000/09/30 20:05:40  napalmbox
// big update
//
// Revision 1.5  2000/09/14 01:19:21  napalmbox
// some fix
//
// Revision 1.4  2000/09/13 19:11:13  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.3.2.3  2000/07/22 19:47:59  napalmbox
// kicq now compiles with -DQT_NO_ASCII_CAST
//
// Revision 1.3.2.2  2000/07/11 02:07:58  napalmbox
// KTabCtl-> QTabWidget ...
//
// Revision 1.3.2.1  2000/07/02 05:45:55  napalmbox
// port kicq -> kde2 :))
//
// Revision 1.3  2000/06/17 17:39:19  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.2  2000/05/29 02:14:39  bills
// minor spelling/grammar fixes
//
// Revision 1.1  2000/05/24 16:59:32  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.16  2000/05/04 15:39:42  denis
// Callbacks moved to the ICQLINK structure.
//
// Revision 1.15  2000/03/30 15:37:25  denis
// Double/int warnings fixed.
//
// Revision 1.14  2000/01/10 16:08:16  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.13  1999/12/02 14:10:53  denis
// Clear widget focus on tab/page switching.
// Cleanups.
//
// Revision 1.12  1999/09/29 17:37:38  denis
// Layout widget stuff implemented.
//
// Revision 1.11  1999/07/16 15:47:55  denis
// Cleaned up.
//
// Revision 1.10  1999/07/16 13:03:46  denis
// Internalization support added.
// Smart loading of spinning face animation implemented.
//
// Revision 1.9  1999/07/12 15:23:19  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.8  1999/05/03 22:17:44  billb
// added sunken border around animation
//
// Revision 1.7  1998/12/08 16:15:49  denis
// Cleaned up a little before releasing
//

#include <icq.h>
#include <qmovie.h>
#include <qmsgbox.h>
#include <qgrpbox.h>
#include <qlayout.h>
#include <kapp.h>

#include "findwindow.h"
#include "mainwindow.h"
#include "userlist.h"
#include "emailwidget.h"
#include "detailswidget.h"
#include "uinwidget.h"

#include "findwindow.moc"

FindWindow::FindWindow() : QDialog()
{

  setCaption(i18n("Search Engine"));
  setIcon(UserIcon("search"));
  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  QBoxLayout *h1Layout = new QHBoxLayout(7);
  topLayout->addLayout(h1Layout);

  tabctl = new KTabCtl(this);
  emailwidget = new EMailWidget(tabctl);
  detailswidget = new DetailsWidget(tabctl);
  uinwidget = new UINWidget(tabctl);
  tabctl->addTab(emailwidget,i18n("EMail"));
  tabctl->addTab(detailswidget,i18n("Details"));
  tabctl->addTab(uinwidget,i18n("ICQ #"));
  connect(tabctl, SIGNAL(tabSelected(int)), SLOT(changeTab(int)));
  h1Layout->addWidget(tabctl, 1);

  QGroupBox *btnbox = new QGroupBox(this);

  QBoxLayout *boxLayout = new QVBoxLayout(btnbox, 7);

  searchbtn = new QPushButton(i18n("Search"),btnbox);
  searchbtn->setFixedHeight(searchbtn->sizeHint().height());
  searchbtn->setDefault(TRUE);
  connect(searchbtn, SIGNAL(clicked()), SLOT(searchCommand()));

  stopbtn = new QPushButton(i18n("Stop"),btnbox);
  stopbtn->setFixedHeight(stopbtn->sizeHint().height());
  stopbtn->setEnabled(FALSE);
  connect(stopbtn, SIGNAL(clicked()), SLOT(stopCommand()));

  newbtn = new QPushButton(i18n("New Search"),btnbox);
  newbtn->setFixedHeight(newbtn->sizeHint().height());
  connect(newbtn, SIGNAL(clicked()), SLOT(clearCommand()));

  int w = QMAX(QMAX(searchbtn->sizeHint().width(), stopbtn->sizeHint().width()),
               newbtn->sizeHint().width());
  searchbtn->setFixedWidth(w);
  stopbtn->setFixedWidth(w);
  stopbtn->setFixedWidth(w);

  boxLayout->addWidget(searchbtn);
  boxLayout->addWidget(stopbtn);
  boxLayout->addWidget(newbtn);

  searchanim = new QLabel(btnbox);
  searchanim->setFixedSize(QSize(32, 32)+QSize(4, 4));
  searchanim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  searchanim->setAlignment(AlignCenter);
  searchanim->setMargin(0);
  QString filename = locate("kicq_pics","search.gif");
  QFile testfile(filename);
  if(!testfile.exists())
    filename = locate("kicq_pics","search.gif");
  searchanim->setMovie(QMovie((const QString &)filename));
  searchanim->movie()->pause();
  searchanim->movie()->restart();
  boxLayout->addWidget(searchanim);
  h1Layout->addWidget(btnbox);

  foundlist = new QListView(this);
  foundlist->addColumn(i18n("ICQ #"));
  foundlist->addColumn(i18n("Handle"));
  foundlist->addColumn(i18n("First Name"));
  foundlist->addColumn(i18n("Last Name"));
  foundlist->addColumn(i18n("EMail"));
  foundlist->addColumn(i18n("Auth"));
  foundlist->setAllColumnsShowFocus(TRUE);
  foundlist->setSorting(0);
  foundlist->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  foundlist->setMultiSelection(TRUE);
  foundlist->setMinimumHeight(100);
  connect(foundlist, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
          SLOT(popupRequest(QListViewItem*, const QPoint&, int)));
  topLayout->addWidget(foundlist, 1);

  popup = new QPopupMenu();
  popup->insertItem(UserIcon("add"), i18n("Add to contact list"), this, SLOT(addRequest()));
  popup->insertItem(UserIcon("info"), i18n("Info"), this, SLOT(infoRequest()));

  readsize();

}

void FindWindow::closeEvent(QCloseEvent *ce)
{
  savesize();
  ce->accept();
}

void FindWindow::resizeEvent(QResizeEvent *re)
{
  QDialog::resizeEvent(re);
  foundlist->setUpdatesEnabled(FALSE);
  foundlist->setColumnWidth(0, (int)(foundlist->width()*0.15));
  foundlist->setColumnWidth(1, (int)(foundlist->width()*0.15));
  foundlist->setColumnWidth(2, (int)(foundlist->width()*0.18));
  foundlist->setColumnWidth(3, (int)(foundlist->width()*0.21));
  foundlist->setColumnWidth(4, (int)(foundlist->width()*0.22));
  foundlist->setColumnWidth(5, (int)(foundlist->width()-
                            foundlist->columnWidth(0)-
                            foundlist->columnWidth(1)-
                            foundlist->columnWidth(2)-
                            foundlist->columnWidth(3)-
                            foundlist->columnWidth(4)-20));
  foundlist->triggerUpdate();
  foundlist->setUpdatesEnabled(TRUE);
  foundlist->repaint();
}

void FindWindow::searchCommand()
{
  if(MW->isLogged())
  {
    if(curtab==0 && !strlen(emailwidget->email().latin1()))
    {
      QMessageBox::warning(this, i18n("Error"), i18n("Empty E-Mail query"));
      return;
    }
    if(curtab==1 && !strlen(detailswidget->nick().latin1()) && !strlen(detailswidget->first().latin1()) &&
       !strlen(detailswidget->last().latin1()))
    {
      QMessageBox::warning(this, i18n("Error"), i18n("Empty Details query"));
      return;
    }
    if(curtab==2 && uinwidget->isEmpty())
    {
      QMessageBox::warning(this, i18n("Error"), i18n("Empty ICQ# query"));
      return;
    }
    searchanim->movie()->unpause();
    searchbtn->setEnabled(FALSE);
    stopbtn->setEnabled(TRUE);
    newbtn->setEnabled(FALSE);
    MW->icqlink->icq_UserFound = userFound;
    MW->icqlink->icq_SearchDone = searchDone;
    if(curtab==0)
      icq_SendSearchReq(MW->icqlink, emailwidget->email().latin1(),"","","");
    else if(curtab==1)
      icq_SendSearchReq(MW->icqlink, "",detailswidget->nick().latin1(),
                        detailswidget->first().latin1(),detailswidget->last().latin1());
    else if(curtab==2)
      icq_SendSearchUINReq(MW->icqlink, uinwidget->uin());
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void FindWindow::stopCommand()
{
  searchanim->movie()->pause();
  if(!isVisible())
    searchanim->movie()->restart();
  searchbtn->setEnabled(TRUE);
  stopbtn->setEnabled(FALSE);
  newbtn->setEnabled(TRUE);
}

void FindWindow::clearCommand()
{
  emailwidget->clear();
  detailswidget->clear();
  uinwidget->clear();
  foundlist->clear();
  foundlist->repaint();
}

void FindWindow::addUserLine(unsigned long uin, const QString &nick, const QString &first,
                             const QString &last, const QString &email, char auth)
{
  QString iuin;
  QListViewItem *tmp;
  iuin.setNum(uin);

  tmp = new QListViewItem(foundlist, (const QString &)iuin, nick, first, last, email, auth==1?i18n("n"):i18n("y"));
}

void FindWindow::userFound(ICQLINK *link, unsigned long uin, const char *nick,
                           const char *first, const char *last, const char *email, char auth)
{
  MW->findWindow()->addUserLine(uin,nick,first,last,email,auth);
}

void FindWindow::searchDone(ICQLINK *link)
{
  MW->findWindow()->stopCommand();

}

void FindWindow::addRequest()
{
  int addFlag = FALSE;
  unsigned long uin;
  QListViewItem *ptr = foundlist->firstChild();
  while(ptr)
  {
    if(foundlist->isSelected(ptr))
    {
      uin = QString(ptr->text(0)).toULong();
      if(!MW->contactList().isUINInContactList(uin))
      {
        ContactItem *item = new ContactItem(uin, ptr->text(1), ptr->text(2), ptr->text(3),
                                            ptr->text(4), ptr->text(5) == i18n("n")?1:0);
        MW->contactList().appendItem(item);
        addFlag = TRUE;
      }
    }
    ptr = ptr->nextSibling();
  }
  if(addFlag)
    MW->updateVisibleList();
}

void FindWindow::infoRequest()
{
  QListViewItem *ptr = foundlist->firstChild();
  while(ptr)
  {
    if(foundlist->isSelected(ptr))
    {
      ContactItem item(QString(ptr->text(0)).toULong(), ptr->text(1), ptr->text(2), ptr->text(3),
                       ptr->text(4), ptr->text(5) == i18n("n")?1:0);
      MW->infoWindow(item);
    }
    ptr = ptr->nextSibling();
  }
}

void FindWindow::popupRequest(QListViewItem *item, const QPoint &pos, int)
{
  if(!item)
    return;
  if(!foundlist->isSelected(item))
  {
    foundlist->setUpdatesEnabled(FALSE);
    QListViewItem *ptr = foundlist->firstChild();
    while(ptr)
    {
      foundlist->setSelected(ptr, FALSE);
      ptr = ptr->nextSibling();
    }
    foundlist->setSelected(item, TRUE);
    foundlist->triggerUpdate();
    foundlist->setUpdatesEnabled(TRUE);
    foundlist->repaint();
  }
  popup->popup(pos);
}

void FindWindow::changeTab(int tab)
{
  QWidget *ptr;
  curtab = tab;
  ptr = emailwidget->focusWidget();
  if(ptr)
    ptr->clearFocus();
  ptr = detailswidget->focusWidget();
  if(ptr)
    ptr->clearFocus();
  ptr = uinwidget->focusWidget();
  if(ptr)
    ptr->clearFocus();
}

void FindWindow::readsize()
{
  db->setPath( MW->getOwnerPath() + "/Geometry/FindWindow" );

  int w = db->readEntry("width").toInt();
  int h = db->readEntry("height").toInt();
  int x = db->readEntry("x").toInt();
  int y = db->readEntry("y").toInt();

  resize(QSize(w, h));
  move(QPoint(x, y));
}

void FindWindow::savesize()
{
  db->setPath( MW->getOwnerPath() + "/Geometry/FindWindow" );

  db->writeEntry("width", this->width());
  db->writeEntry("height", this->height());
  db->writeEntry("x", this->x());
  db->writeEntry("y", this->y());

  db->flush();
}

