/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _ICQMESSAGE_
#define _ICQMESSAGE_

#include <qstring.h>
#include <qfile.h>
#include <qdatastream.h>

enum icqMsgType { MSG_UNDEF, MSG_SENT, MSG_RECV, MSG_URL_SENT, MSG_URL_RECV,
                  MSG_AUTH_RECV, MSG_AUTH_GRANT, MSG_AUTH_REFUSE, MSG_CHAT_REQ_RECV,
                  MSG_ADDED, MSG_PAGER_RECV, MSG_EXPRESS_RECV, MSG_FILE_REQ_RECV };

typedef struct TmStr
{
  unsigned char hour;
  unsigned char minute;
  unsigned char day;
  unsigned char month;
  unsigned short year;
  TmStr(void)
  {
    hour = 0;
    minute = 0;
    day = 0;
    month = 0;
    year = 0;
    return;
  }
  TmStr(unsigned char phour, unsigned char pmin, unsigned char pday,
        unsigned char pmonth, unsigned short pyear)
  {
    hour = phour;
    minute = pmin;
    day = pday;
    month = pmonth;
    year = pyear;
    return;
  }
} TimeStr;

class icqMessage
{
public:
  icqMessage(void) { return; }
  /* Message */
  icqMessage(unsigned long, TimeStr, const QString &, icqMsgType);
  icqMessage(unsigned long, TimeStr, const QString &, unsigned long, icqMsgType);
  /* URL */
  icqMessage(unsigned long, TimeStr, const QString &, const QString &, icqMsgType);

  /* AuthRecv */
  icqMessage(unsigned long, TimeStr, const QString &, const QString &, const QString &,
             const QString &, const QString &, icqMsgType);
  /* AuthGrant/AuthRefuse/Added */
  icqMessage(unsigned long, TimeStr, const QString &, const QString &, const QString &,
             const QString &, icqMsgType);
  /* Pager/Express */
  icqMessage(unsigned long, TimeStr, const QString &, const QString &, const QString &,
             icqMsgType);
  /* File Request */
  icqMessage(unsigned long, TimeStr, const QString &, const QString &, unsigned long,
             unsigned long, icqMsgType);
  icqMessage(icqMessage*);
  const QString body(void) const  { return msgBody; }
  unsigned long uin(void) const   { return msgUin;  }
  const QString url(void) const   { return msgURL;  }
  const QString nick(void) const  { return msgNick; }
  const QString first(void) const { return msgFirst;}
  const QString last(void) const  { return msgLast; }
  const QString email(void) const { return msgEmail;}
  const unsigned long seq(void) const { return msgSeq; }
  const QString typeString(void)      { return icqMsgTypeStrings[msgType]; }
  const QString typeTitleString(void) { return icqMsgTypeTitles[msgType]; }
  const QString file(void) const      { return msgFile; }
  unsigned long fileSize(void) const  { return msgFileSize; }
  static const QString typeString(icqMsgType t) { return icqMsgTypeStrings[t]; }
  TimeStr time(void) const { return msgTime; }
  int isNew(void) const { return msgNew; }
  void expire(void);
  icqMsgType type(void) { return msgType; }
  void setType(icqMsgType);
  void save(void);
  bool load(Q_UINT32 n);
protected:
  icqMsgType msgType;
  int msgNew;
  unsigned long msgUin;
  TimeStr msgTime;
  QString msgBody;
  QString msgURL;
  QString msgNick;
  QString msgFirst;
  QString msgLast;
  QString msgEmail;
  QString msgFile;
  unsigned long msgFileSize;
  unsigned long msgSeq;
  static const QString icqMsgTypeStrings[];
  static const QString icqMsgTypeTitles[];
};
#endif


