/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: docking.cpp,v 1.6 2001/05/25 00:50:12 denis Exp $
// Revision 1.1.2.3  2000/07/10 11:24:33  napalmbox
// dockwidget use KSystemTray now - work with lates kde2 cvs
//
// Revision 1.1.2.2  2000/07/02 10:29:04  napalmbox
// up2date
//
// Revision 1.1.2.1  2000/07/02 05:45:55  napalmbox
// port kicq -> kde2 :))
//
// Revision 1.1  2000/05/24 16:59:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.12  2000/05/05 03:37:49  bills
// kicq will now both dock and undock when double clicking on dock icon.  in
// addition, it will be automatically brought to the current desktop, if
// necessary
//
// Revision 1.11  2000/01/10 16:08:11  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.10  1999/12/13 04:58:32  kyle
//
// docking.cpp - double-click on dock icon will now raise kicq window
//
// Revision 1.9  1999/12/12 18:08:06  denis
// Existing kapp->topWidget()'s chnaged to MW macros.
//
// Revision 1.8  1999/12/02 13:59:06  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:47:49  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:01:46  denis
// Internalization support added.
//
// Revision 1.5  1998/12/08 16:15:03  denis
// setToolTip added
// Cleaned up a little before releasing
//

#include <qtooltip.h>
#include <kwin.h>
#include <kapp.h>
#include <kaction.h>

#include "mainwindow.h"
#include "docking.h"

#include "docking.moc"

DockWidget::DockWidget(QWidget *parent, const char *name) : KSystemTray(parent,name)
{
  docked = FALSE;
  doRemove = FALSE;
  dockPopup = new KPopupMenu();
  dockPopup->insertTitle(i18n("kicq"));
  toggleID = dockPopup->insertItem(i18n("Restore"), this, SLOT(showWindow()));
  dockPopup->insertItem(i18n("Status"), MW->stateMenu());
  dockPopup->insertSeparator();
  dockPopup->insertItem(UserIcon("setup"), i18n("Preferences..."), MENU_PREFERENCES);
  dockPopup->insertItem(UserIcon("help"), i18n("Help"), MW->helpMenu(QString::null, false));
  dockPopup->insertItem(UserIcon("shutdown"), i18n("Shutdown"), this, SLOT(emitQuit()));
  QToolTip::add(this, MW->name());
  connect(dockPopup, SIGNAL(activated(int)), MW, SLOT(icqCommand(int)));
  return;
}

DockWidget::~DockWidget(void)
{
  return;
}

void DockWidget::dock(void)
{
  if(docked)
    return;
  KWin::setSystemTrayWindowFor(this->winId(), winId());
  this->setFixedSize(24, 24);
  show();
  paintIcon();
  update();
  docked = TRUE;
  return;
}

void DockWidget::undock(void)
{
  if(!docked)
    return;
  // the widget's window has to be destroyed in order 
  // to undock from the panel. Simply using hide() is
  // not enough.
  this->destroy(TRUE, TRUE);
  // recreate window for further dockings
  this->create(0, TRUE, FALSE);
  docked = FALSE;
  return;
}

const int DockWidget::isDocked(void)
{
  return docked;
}

void DockWidget::paintEvent(QPaintEvent*)
{
  paintIcon();
}

void DockWidget::paintIcon(void)
{
  bitBlt(this, (24-dockIcon.width())/2, (24-dockIcon.height())/2, &dockIcon);
  return;
}

void DockWidget::mouseDoubleClickEvent(QMouseEvent *e)
{
  if(e->button() == LeftButton)
  {
    if(MW->isUnreadMsgs())
      MW->readFirstRequest();
    else 
    {
      if(MW->isVisible())
        hideWindow();
      else
        showWindow();
//    KWin::setCurrentDesktop(KWin::currentDesktop());
    }
  }
  return;
}

void DockWidget::mousePressEvent(QMouseEvent *e)
{
  if(e->button() == RightButton || e->button() == MidButton)
  {
    int x = e->x();
    int y = e->y();
    QString text;
    if(MW->isUnreadMsgs())
    {
      if(!doRemove)
      {
        int readID = dockPopup->insertItem(i18n("Read Message"), -1, 0);
        dockPopup->connectItem(readID, MW, SLOT(readFirstRequest()));
        dockPopup->insertSeparator(1);
        doRemove = TRUE;
      }
    }
    else
      if(doRemove)
      {
        dockPopup->removeItemAt(1);
        dockPopup->removeItemAt(0);
        doRemove = FALSE;
      }
    if(MW->isVisible())
    {
      text = i18n("Dock");
      dockPopup->disconnectItem(toggleID, this, SLOT(showWindow()));
      dockPopup->connectItem(toggleID, this, SLOT(hideWindow()));
    }
    else
    {
      text = i18n("Restore");
      dockPopup->disconnectItem(toggleID, this, SLOT(hideWindow()));
      dockPopup->connectItem(toggleID, this, SLOT(showWindow()));
    }
    dockPopup->changeItem(text, toggleID);
    dockPopup->popup(mapToGlobal(QPoint(x, y)));
    dockPopup->exec();
  }
  return;
}

void DockWidget::hideWindow(void)
{
  if(MW->isVisible())
    MW->hide();
  return;
}

void DockWidget::showWindow(void)
{
  if(!MW->isVisible())
    MW->show();
  MW->raise();
  KWin::setCurrentDesktop(KWin::currentDesktop());
  return;
}

void DockWidget::emitQuit(void)
{
  emit quitClicked();
  return;
}

void DockWidget::setIcon(QPixmap icon)
{
  dockIcon = icon;
  update();
  return;
}

void DockWidget::setToolTip(QString text)
{
  QToolTip::remove(this);
  QToolTip::add(this, (const QString &)text);
  return;
}
